<?php

require_once(KT_LIB_DIR . '/plugins/plugin.inc.php');
require_once(KT_LIB_DIR . '/actions/documentaction.inc.php');
require_once(KT_DIR . '/plugins/ktcore/scheduler/schedulerUtil.php');
require_once('_config.php');
/**
 * Document action that allows the actual viewing of a document
 *
 */
class SecViewDocumentAction extends KTDocumentAction
{
	// The namespace for this action
	var $sName = 'ktcore.SecViewPlugin.actions.document';

	// The permission required to see this action on the menu
	var $sPermissionName = 'ktcore.permissions.read';

	var $aAcceptedMimeTypes = array('doc', 'ods', 'odt', 'ott', 'txt', 'rtf', 'sxw', 'stw',
	'xml' , 'pdb', 'psw', 'ods', 'ots', 'sxc',
	'stc', 'dif', 'dbf', 'xls', 'xlt', 'slk', 'csv', 'pxl',
	'odp', 'otp', 'sxi', 'sti', 'ppt', 'pot', 'sxd', 'odg',
	'otg', 'std', 'asc','pdf');

	var $aImageMimeTypes = array('png','jpg','gif','bmp','jpeg','jpe');

	// Store here alternative output path
	var $sOverridePath;

	/**
	 * Method for getting the MIME type extension for the current document.
	 * Source: KnowledgeTree Community Edition v3.5.2c
	 *
	 * @return string mime time extension
	 */
	private function getMimeExtension()
	{

		if($this->oDocument == null || $this->oDocument == "" || PEAR::isError($this->oDocument) ) return _kt('Unknown Type');

		$oDocument = $this->oDocument;
		$iMimeTypeId = $oDocument->getMimeTypeID();
		$mimetypename = KTMime::getMimeTypeName($iMimeTypeId); // mime type name

		$sTable = KTUtil::getTableName('mimetypes');
		$sQuery = "SELECT filetypes FROM " . $sTable . " WHERE mimetypes = ?";
		$aQuery = array($sQuery, array($mimetypename));
		$res = DBUtil::getResultArray($aQuery);
		if (PEAR::isError($res))
		{
			return $res;
		}
		else
		{
			if (count($res) != 0)
			{
				return $res[0]['filetypes'];
			}
		}
		return _kt('Unknown Type');
	}

	function getName()
	{
		return _kt("SecView");
	}

	/**
	 * Returns display name for use in document sidebar.
	 *
	 * @return display name
	 */
	function getDisplayName()
	{
		$cmdpath = KTUtil::findCommand('externalBinary/python');
		// Check if openoffice and python are available
		if($cmdpath != false && file_exists($cmdpath) && !empty($cmdpath))
		{
			$sDocType = $this->getMimeExtension();
			// make sure that the selected document is of an acceptable extension
			foreach(array_merge($this->aAcceptedMimeTypes,$this->aImageMimeTypes) as $acceptType)
			{
				if($acceptType == $sDocType)
				{
					return '<b>' . _kt('SecView', 'plugins')."</b> <img src=\"".SECVIEW_PLUGIN_PATH."img/edit-find.png\" />";
				}
			}
		}
		return '';
	}

	public function getAvailability()
	{

	}

	private function displayImage()
	{
		return "<img src=\"".SECVIEW_PLUGIN_PATH."getDoc.php?doc=".$this->oDocument->getId()."\" />";
	}

	public function getPdfTarget() {
		if (empty($this->sOverridePath)) {
			return SECVIEW_REPOSITORY_PATH . $this->oDocument->getId()."/pdf/".$this->oDocument->getId().".pdf";
		} else {
			return $this->sOverridePath;
		}
	}

	public function setPdfTarget($sPath) {
		$this->sOverridePath = $sPath;
	}

	/**
	 * Converts document into an acceptable format for the viewer, and loads the document into the viewer.
	 *
	 * @return unknown
	 */
	function do_main()
	{
		$props = array();
		
		// проверка прав на просмотр документа
		if(!Permission::userHasDocumentSecviewPermission($this->oDocument)){
			$this->addErrorMessage(_kt('У Вас нет прав на online просмотр этого документа'));
			redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$this->oDocument->getId())));
			exit(0);
		}

		//check if we're dealing with an image
		$sDocType = $this->getMimeExtension();
		foreach($this->aImageMimeTypes as $acceptType){
			if($acceptType == $sDocType){
				return $this->displayImage();
			}
		}

		//check if file type is supported
		$accept = false;
		foreach($this->aAcceptedMimeTypes as $acceptType){
			if($acceptType == $sDocType){
				$accept = true;
			}
		}
		
		if($accept == false){
			$this->addErrorMessage(_kt('An error occurred viewing the document - document type not supported'));
			redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$this->oDocument->getId())));
			exit(0);
		}

		//Retrieve properties of converted document.
		if(file_exists(SECVIEW_REPOSITORY_PATH . $this->oDocument->getId()."/".$this->oDocument->getId().".prop")){
			$props = unserialize(file_get_contents(SECVIEW_REPOSITORY_PATH . $this->oDocument->getId()."/".$this->oDocument->getId().".prop"));
		}

		//check if the document already has been converted and if the converted document is still the most recent version.
		if(!file_exists(SECVIEW_REPOSITORY_PATH.$this->oDocument->getId()."/".$this->oDocument->getId().".prop")
			|| (isset($props['content_version']) && $props['content_version'] != $this->oDocument->getContentVersionId())
		)
		{
			require_once("PdfToImagesConverter.php");
			$PdftoImagesConverter = new PdftoImagesConverter();
			$props['content_version'] = $this->oDocument->getContentVersionId();
			$PdftoImagesConverter->ClearDocumentRepository($this->oDocument->getId());

			if(!is_dir($PdftoImagesConverter->repository_path.$this->oDocument->getId())){
				@mkdir($PdftoImagesConverter->repository_path.$this->oDocument->getId());
			}
			$this->createPdf();
			$props['page_count'] = $PdftoImagesConverter->getPDFPageCount($PdftoImagesConverter->repository_path.$this->oDocument->getId().'/pdf/'.$this->oDocument->getId().".pdf");

			@unlink($PdftoImagesConverter->repository_path.$this->oDocument->getId().'/'.$this->oDocument->getId().".prop");
			file_put_contents($PdftoImagesConverter->repository_path.$this->oDocument->getId().'/'.$this->oDocument->getId().".prop",serialize($props));
		}

		//Get the properties of the current document.
		$props = unserialize(file_get_contents(SECVIEW_REPOSITORY_PATH.$this->oDocument->getId()."/".$this->oDocument->getId().".prop"));

		// create the document transaction record
        include_once(KT_LIB_DIR . '/documentmanagement/DocumentTransaction.inc');
        $oDocumentTransaction = & new DocumentTransaction($this->oDocument, "Document secviewed", 'ktcore.transactions.secview');
        $oDocumentTransaction->iUserID = $this->oUser->iId;
        $oDocumentTransaction->create();
                
		//show the viewer and load the document.
		global $main;
		$oJSON = new Services_JSON();
		$settings = $oJSON->decode(KTUtil::getSystemSetting("sec-view-plugin-settings"));

		$main->requireCSSresource( SECVIEW_PLUGIN_PATH . "css/SecView-main.css");
		$main->requireCSSresource( SECVIEW_PLUGIN_PATH . "css/SecView-main-ie.css", true);

		$main->requireJSResource("thirdpartyjs/extjs/adapter/prototype/prototype.js");
		$main->requireJSResource(SECVIEW_PLUGIN_PATH . "js/SecViewPlugin.js");

		$oTemplating =& KTTemplating::getSingleton();
		$oTemplate = $oTemplating->loadTemplate('SecView/SecView');
		$aTemplateData = array(
			"document_id"=>$this->oDocument->getId(),
			"plugin_path"=>SECVIEW_PLUGIN_PATH,
			"page_count"=>intval($props["page_count"]),
			"plugin_width"=>intval($settings->plugin_width),
			"plugin_height"=>intval($settings->plugin_height)
		);
		$aTemplateData['i18n_domain'] ='plugins';
		return $oTemplate->render($aTemplateData);
	}
	
	public function createPdf()
	{
		if(file_exists($this->getPdfTarget())){
			@unlink($this->getPdfTarget());
		}
		$oDocument = $this->oDocument;
		$oStorage =& KTStorageManagerUtil::getSingleton();
		$oConfig =& KTConfig::getSingleton();
		$cmdpath = KTUtil::findCommand('externalBinary/python');

		//path to document on FS
		$sPath = sprintf("%s/%s", $oConfig->get('urls/documentRoot'), $oStorage->getPath($oDocument));

		//We don't have to convert pdfs to pdf since.... they're a pdf already ;-)
		if($this->getMimeExtension() != 'pdf'){
			// Check if openoffice and python are available
			if($cmdpath == false || !file_exists($cmdpath) || empty($cmdpath)){
				// Set the error messsage and redirect to view document
				$this->addErrorMessage(_kt('An error occurred converting the document to PDF - please contact the system administrator. Python binary not found.'));
				redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$oDocument->getId())));
				exit(0);
			}

			//does the document exist on the server?
			if (file_exists($sPath)){

				//convert document to PDF, code taken from KT CE 3.5.2c

				// Get a tmp file
				$sTempFilename = tempnam('/tmp', 'ktpdf');

				// We need to handle Windows differently
				if (substr( PHP_OS, 0, 3) == 'WIN'){
					$cmd = "\"" . $cmdpath . "\" \"". KT_DIR . "/bin/openoffice/pdfgen.py\" \"" . $sPath . "\" \"" . $sTempFilename . "\"";
					$cmd = str_replace( '/','\\',$cmd);
					$res = `"$cmd" 2>&1`;
				} else {
					$cmd = $cmdpath . ' ' . KT_DIR . '/bin/openoffice/pdfgen.py ' . escapeshellcmd($sPath) . ' ' . escapeshellcmd($sTempFilename);
					$res = shell_exec($cmd." 2>&1");
				}


				if (file_exists($sTempFilename)){
					if(!is_dir(dirname($this->getPdfTarget()))){
						@mkdir(dirname($this->getPdfTarget()));
					}
					copy($sTempFilename,$this->getPdfTarget());
				} else {
					$this->addErrorMessage(_kt('An error occurred converting the document to PDF - please contact the system administrator. Python binary not found.'));
					redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$oDocument->getId())));
					exit(0);
				}
			} else {
				$this->addErrorMessage(_kt('An error occurred viewing this file - file not found'));
				redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$oDocument->getId())));
				exit(0);
			}
		} else {
			//Document is PDF already.
			if(!is_dir(SECVIEW_REPOSITORY_PATH . $this->oDocument->getId()."/pdf")) {
				@mkdir(SECVIEW_REPOSITORY_PATH . $this->oDocument->getId()."/pdf");
			}
			copy($sPath ,$this->getPdfTarget());
		}
		return true;
	}
}

class SecViewAdminDocumentAction extends KTDocumentAction
{
	// The namespace for this action
	var $sName = 'ktcore.SecViewPlugin.admin.actions.document';

	// The permission required to see this action on the menu
	var $sPermissionName = 'ktcore.permissions.write';

	function getName()
	{
		return _kt("SecView Hash Status");
	}

	/**
	 * Returns display name for use in document sidebar.
	 *
	 * @return display name
	 */
	function getDisplayName()
	{
		if (Permission::userIsSystemAdministrator() && $_SESSION['adminmode']){
			if(!is_object($this->oDocument)){
				return '';
			}
			if (Indexer::isDocumentScheduled($this->oDocument->getId())){
				return false;
			}else{
				return _kt('SecView Hash Status');
			}
		}
		return '';
	}

	private function displayImage()
	{
		return true;
	}



	function do_main()
	{
		$doc=$this->oDocument;
		$docid=$doc->getId();
		if (Permission::userIsSystemAdministrator()) {
			require_once(KT_DIR . '/' . SECVIEW_PLUGIN_PATH."PdfToImagesConverter.php");
			$PdftoImagesConverter = new PdftoImagesConverter();
			$full_path = $doc->getFullPath();

			$task = schedulerEntity::getByTaskName("SecView Cron ".$doc->getId());
			if(PEAR::isError($task)){
				$DocumentShowRun = true;
			}else{
				$DocumentShowRun = false;
			}

			$props = unserialize(file_get_contents(SECVIEW_REPOSITORY_PATH.$doc->getId()."/".$doc->getId().".prop"));

			$Plugin_Size = array("small","middle","large");
			$Plugin_Quality = array("fast","slow");
			$hash_count = 0;
			for($i = 0;$i<intval(@$props["page_count"]);$i++){
				foreach($Plugin_Size as $size){
					foreach($Plugin_Quality as $quality){
						if($PdftoImagesConverter->isHashExist($doc->getId(),$i,$size,$quality) || $create_Hash){
							$hash_count++;
						}
					}
				}
			}

			global $main;

			$main->requireCSSresource( SECVIEW_PLUGIN_PATH . "css/SecView-admin.css");

			$oTemplating =& KTTemplating::getSingleton();
			$oTemplate = $oTemplating->loadTemplate('SecView/SecViewAdmin');
			$aTemplateData = array(
				"document_id"=>$docid,
				"plugin_path"=>SECVIEW_PLUGIN_PATH,
				"FullPath"=>$full_path,
				"HashCount"=>$hash_count,
				"DocumentShowRun"=>$DocumentShowRun,
				"DocumentPageCount"=>intval($props['page_count']),
				'page_url'=>$_SERVER['PHP_SELF'],
			);
			$aTemplateData['i18n_domain'] ='plugins';
			return $oTemplate->render($aTemplateData);

			return '';


		}
		redirect("view.php?fDocumentId=$docid");
		exit;
	}

	function do_runCron()
	{
		$doc=$this->oDocument;
		$id=$doc->getId();

		schedulerUtil::registerBackgroundTask('SecView Cron '.$id, 'plugins/SecView/cron/cronSecView.php',array('id'=>$id));
		redirect("view.php?fDocumentId=$id");
		exit;
	}
	function do_clearHash()
	{
		$doc=$this->oDocument;
		$id=$doc->getId();
		require_once(KT_DIR . '/' . SECVIEW_PLUGIN_PATH."PdfToImagesConverter.php");
		$PdftoImagesConverter = new PdftoImagesConverter();
		$PdftoImagesConverter->ClearDocumentRepository($id);
		redirect("view.php?fDocumentId=$id");
		exit;
	}
}

/**
 * This plugin allows online viewing of all kinds of documents.
 */
class secView extends KTPlugin
{
	//namespamce
	var $sNamespace = "ktcore.SecView.plugin";

	/**
	 * Construct plugin
	 *
	 * @param string $sFilename
	 * @return secView plugin object
	 */
	function secView($sFilename = null)
	{
		$res = parent::KTPlugin($sFilename);
		$this->sFriendlyName = _kt('SecView Plugin');
		return $res;
	}

	/**
	 * Setup template path, actions, admin pages etc.
	 *
	 */
	function setup()
	{
		//set up swf dir
		if(!is_dir(SECVIEW_REPOSITORY_PATH)){
			die("Please create the directory ".SECVIEW_REPOSITORY_PATH." first in the way described in the installation instructions of this plugin.");
		}

		//create log file
		if(!is_file(SECVIEW_ODV_LOG_FILE)){
			if($fh = @fopen(SECVIEW_ODV_LOG_FILE,"a+b")){
				fclose($fh);
			}
			if(!is_file(SECVIEW_ODV_LOG_FILE)){
				die("Can't create log file: " . SECVIEW_ODV_LOG_FILE.". Please verify if the containing directory exists and permissions are set correctly");
			}
		}

		$sPermissionNameDenyDownload = 'ktcore.permissions.denydownload';
		$oPermissionDenyDownload =& KTPermission::getByName($sPermissionName);
		$sPermissionNameDenySecview = 'ktcore.permissions.denysecview';
		$oPermissionDenySecview =& KTPermission::getByName($sPermissionName);

		if (PEAR::isError($oPermissionDenyDownload)) {
			$oPermissionDenyDownload =& KTPermission::createFromArray(array(
				'name' => $sPermissionNameDenyDownload,
				'humanname' => _kt('Deny Download', 'plugins')
			));
		}
		
		if (PEAR::isError($oPermissionDenySecview)) {
			$oPermissionDenySecview =& KTPermission::createFromArray(array(
				'name' => $sPermissionNameDenySecview,
				'humanname' => _kt('Deny Secview', 'plugins')
			));
		}
		
		//Document action
		$this->registerAction('documentaction', 'SecViewDocumentAction', 'ktcore.SecViewPlugin.actions.document');
		$this->registerAction('documentaction', 'SecViewAdminDocumentAction', 'ktcore.SecViewPlugin.admin.actions.document');

		$this->registerAdminCategory("SecViewManagement", _kt("SecView Management"),
		_kt("View Documents online"));

		$this->registerAdminPage("configure",'ConfigureSecViewDispatcher','SecViewManagement', _kt('SecView settings'),
		_kt('Set Online Viewer Settings.'), 'admin/configure.php');

		$this->registerAdminPage("DocumentsStatus",'DocumentsStatusSecViewDispatcher','SecViewManagement', _kt('SecView Documents Status'),
		_kt('Documents Status'), 'admin/documentsStatus.php');

		require_once(KT_LIB_DIR . "/templating/templating.inc.php");
		$oTemplating =& KTTemplating::getSingleton();
		$oTemplating->addLocation('SecView Plugin', '/'.SECVIEW_PLUGIN_PATH.'templates');


		$oJSON = new Services_JSON();

		$settings = new stdClass();
		$settings->quality_small = 70;
		$settings->resize_small = '800x800';
		$settings->colorspace_small = 'RGB';
		$settings->density_small = 300;

		$settings->quality_middle = 70;
		$settings->resize_middle = '1000x1000';
		$settings->colorspace_middle = 'RGB';
		$settings->density_middle = 300;

		$settings->quality_large = 70;
		$settings->resize_large = '1200x1200';
		$settings->colorspace_large = 'RGB';
		$settings->density_large = 300;

		$settings->quality_small_slow = 50;
		$settings->resize_small_slow = '800x800';
		$settings->colorspace_small_slow = 'RGB';
		$settings->density_small_slow = 300;

		$settings->quality_middle_slow = 50;
		$settings->resize_middle_slow = '1000x1000';
		$settings->colorspace_middle_slow = 'RGB';
		$settings->density_middle_slow = 300;

		$settings->quality_large_slow = 50;
		$settings->resize_large_slow = '1200x1200';
		$settings->colorspace_large_slow = 'RGB';
		$settings->density_large_slow = 300;

		$settings->convert_path		= 'convert';
		$settings->identify_path	= 'identify';

		$settings->plugin_creation_possibility = 1;
		$settings->plugin_width = 800;
		$settings->plugin_height = 800;

		$state = $oJSON->encode($settings);

		KTUtil::setSystemSetting("sec-view-plugin-settings",$state);

		schedulerUtil::registerTask(
			'SecView Create Hash',					//name
			'plugins/SecView/cron/cronSecView.php',  //path
			'',									//params
			'daily',							//frequency
			date("Y-m-d H:i:s",mktime(date("H"), date("i")+10, date("s"), date("m"), date("d")-1, date("Y"))),				//StartTime
			'enabled'							//Status
		);
		schedulerUtil::registerTask(
			'SecView Clear Hash',					//name
			'plugins/SecView/cron/cronClearHash.php',  //path
			'',									//params
			'daily',							//frequency
			date("Y-m-d H:i:s",mktime(date("H"), date("i")+10, date("s"), date("m"), date("d")-1, date("Y"))),				//StartTime
			'enabled'							//Status
		);
	}
}

$oRegistry =& KTPluginRegistry::getSingleton();
$oRegistry->registerPlugin('SecView', 'ktcore.SecView.plugin', __FILE__);

?>