<?php
/**
 * @author Aart-Jan Boor <aart-jan@wemag.nl>
 * @copyright Aart-Jan Boor <aart-jan@wemag.nl>
 *
 * This file is part of Wemag Online Document Viewer.
 *
 * Wemag Online Document Viewer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wemag Online Document Viewer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Wemag Online Document Viewer.  If not, see <http://www.gnu.org/licenses/>.
 */



/**
 * Class to convert PDFs to SWF movieclips
 *
 */

class PdfToSwfConverter{
	/**
	 * Writes log to a log file
	 *
	 * @param string $txt log text
	 * @param file $logfile log file
	 */
	private static function writeLog($txt,$logfile = null){
		if($logfile == null){
			$logfile = KT_DIR.WEMAG_ODV_LOG_FILE;
		}
		if(@filesize($logfile) > 10000){
			@unlink($logfile);
		}
		file_put_contents($logfile,date('r').":".$txt."\n",FILE_APPEND);
	}
	
	/**
	 * Converts a PDF to an SWF
	 *
	 * @param file $src src file
	 * @param file $destination destination file
	 * @param array $extra_props extra properties for companying property file
	 */
	public static function convertPdfToSwf($src, $destination, $extra_props = array()){
		$tempname = time();
		if(!is_file($src) || filesize($src) == 0){
			die("An error occured: Invalid source file (PDF conversion error - corrupt source file) ".$src);
		}
		 if (substr( PHP_OS, 0, 3) == 'WIN') {
			PdfToSwfConverter::writeLog(exec(KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/pdf2swf.exe $src $destination"));
		 }else{
		 	PdfToSwfConverter::writeLog(exec("pdf2swf $src $destination"));
		 }
		$output = array();
		if (substr( PHP_OS, 0, 3) == 'WIN') {
			PdfToSwfConverter::writeLog(exec(KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/swfdump.exe $destination",$output));
		}else{
			PdfToSwfConverter::writeLog(exec("swfdump $destination",$output));
		}
		$width = 500; 
		$height = 800;
		for($i=0; $i<=20; $i++){
			if(substr($output[$i],0,29) == "[HEADER]        Movie width: "){
				$end = strpos($output[$i],".");
				$width = substr($output[$i],29,$end-29);
			}
			if(substr($output[$i],0,30) == "[HEADER]        Movie height: "){
				$end = strpos($output[$i],".");
				$height = substr($output[$i],30,$end-30);
			}
		}
		if (substr( PHP_OS, 0, 3) == 'WIN') {
			PdfToSwfConverter::writeLog(exec(KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/swfcombine.exe -X $width -Y $height \"".KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/swfs/wemag_viewer.swf\" viewport=\"$destination\" -o \"$destination\""));
		}else{
			PdfToSwfConverter::writeLog(exec("swfcombine -X $width -Y $height \"".KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/swfs/wemag_viewer.swf\" viewport=\"$destination\" -o \"$destination\""));
		}
		//PdfToSwfConverter::writeLog(exec(KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/swfcombine.exe -X $width -Y $height -r 20.000000 \"".KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/swfs/PreLoaderTemplate.swf\" loader=\"".KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/swfs/default_loader.swf\" movie=\"$destination\" -o \"$destination\""));
		//unlink(KT_DIR."/plugins/WemagOnlineDocumentViewer/pdf2swf/temp/".$tempname.".swf");

		PdfToSwfConverter::writeLog("\n========================================\n\n");

		//delete old property file
		@unlink($destination.".prop");
		
		//write property file
		file_put_contents($destination.".prop",serialize(array_merge(array('width'=>$width,'height'=>$height),$extra_props)));
	}
	
	
}
?>
