<?PHP
require_once "Structures/DataGrid.php";
require_once(KT_DIR . "/thirdparty/Smarty/Smarty.class.php");
require_once(KT_LIB_DIR . "/templating/template.inc.php");
require_once(KT_LIB_DIR . "/i18n/i18nregistry.inc.php");

class LMCommon {

	/**
    * Получить стандартную заготовку для таблицы
    *
    * @return Structures_DataGrid
    */
	function GetEmptyDataGrid($limit = 3, $page = null, $renderer = DATAGRID_RENDERER_TABLE) {

		// Define New DataGrid with a limit of 3 records
		$dg = & new Structures_DataGrid($limit, $page, $renderer);

		// Define DataGrid Color Attributes
		//$dg->renderer->setTableHeaderAttributtes(array('bgcolor' => '#FFFF66'));

		if ($renderer == DATAGRID_RENDERER_TABLE) {
			$dg->renderer->setTableOddRowAttributes(array('id' => 'odd'));
			$dg->renderer->setTableEvenRowAttributes(array('id' => 'even'));

			// Define DataGrid Table Attributes
			$dg->renderer->setTableAttribute('width', '100%');
			$dg->renderer->setTableAttribute('cellspacing', '1px');
			$dg->renderer->setTableAttribute('cellpadding', '4');
			$dg->renderer->setTableAttribute('class', 'datagrid');

			// Set empty row table attributes
			$dg->renderer->allowEmptyRows(false, array('id' => 'empty'));
		}

		$dg->renderer->sortIconASC = "&uArr;";
		$dg->renderer->sortIconDESC = "&dArr;";

		return $dg;
	}

	function GetSmarty($aDict = null) {
		$smarty = & new Smarty();
		$smarty->compile_dir = "/tmp";
		$oConfig =& KTConfig::getSingleton();
		$sVarDirectory = $oConfig->get('urls/varDirectory');

		foreach (array($sVarDirectory . '/tmp', '/tmp') as $sPath) {
			if (is_writeable($sPath)) {
				$smarty->compile_dir = $sPath;
				break;
			}
		}
		if (is_array($aDict)) {
			$iLen = count($aDict);
			$aKeys = array_keys($aDict);
			for ($i = 0; $i < $iLen; $i++) {
				$sKey = $aKeys[$i];
				$smarty->assign_by_ref($sKey, $aDict[$sKey]);
			}
		}
		if (is_array($this->aDict)) {
			$iLen = count($this->aDict);
			$aKeys = array_keys($this->aDict);
			for ($i = 0; $i < $iLen; $i++) {
				$sKey = $aKeys[$i];
				$smarty->assign_by_ref($sKey, $this->aDict[$sKey]);
			}
		}

		$KTConfig =& KTConfig::getSingleton();

		// needed for a very, very few places.
		$isSSL = $KTConfig->get("KnowledgeTree/sslEnabled");
		$hostname = $KTConfig->get("KnowledgeTree/serverName");
		$absroot = 'http';
		$absroot .= ($isSSL) ? 's://' : '://';
		$absroot .= $hostname;
		$absroot .= $KTConfig->get("KnowledgeTree/rootUrl");

		$smarty->assign("config", $KTConfig);
		$smarty->assign("rootUrl", $KTConfig->get("KnowledgeTree/rootUrl"));
		$smarty->assign("absoluteRootUrl", $absroot);
		$smarty->caching = false;
		$smarty->register_function('entity_select', array('KTSmartyTemplate', 'entity_select'));
		$smarty->register_function('boolean_checkbox', array('KTSmartyTemplate', 'boolean_checkbox'));
		$smarty->register_function('entity_checkboxes', array('KTSmartyTemplate', 'entity_checkboxes'));
		$smarty->register_function('entity_radios', array('KTSmartyTemplate', 'entity_radios'));
		$smarty->register_block('i18n', array('KTSmartyTemplate', 'i18n_block'), false);
		$smarty->register_modifier('addQueryString', array('KTSmartyTemplate', 'addQueryString'));
		$smarty->register_function('ktLink', array('KTSmartyTemplate', 'ktLink'));
		$smarty->register_modifier('addQS', array('KTSmartyTemplate', 'addQueryString'));
		$smarty->register_modifier('addQueryStringSelf', array('KTSmartyTemplate', 'addQueryStringSelf'));
		$smarty->register_modifier('addQSSelf', array('KTSmartyTemplate', 'addQueryStringSelf'));
		$smarty->register_block('addQS', array('KTSmartyTemplate', 'addQueryStringBlock'), false);
		$smarty->register_function('getUrlForFolder', array('KTSmartyTemplate', 'getUrlForFolder'));
		$smarty->register_function('getCrumbStringForDocument', array('KTSmartyTemplate', 'getCrumbStringForDocument'));

		return $smarty;
	}

	function _i18n_get_args($arr, $var) {
		if (!is_array($arr)) {
			$arr = array();
		}
		if (substr($var[0], 0, 4) == "arg_") {
			$arr['#' . substr($var[0], 4) . '#'] = $var[1];
		}
		return $arr;
	}

	function i18n_block($params, $content, &$smarty, &$repeat) {
		if (empty($content)) {
			return;
		}
		if (!empty($params)) {
			$flattened = array_map(null, array_keys($params), array_values($params));
			$replacements = array_reduce($flattened, array('KTSmartyTemplate', '_i18n_get_args'), "");
		} else {
			$replacements = array();
		}
		$sDomain = KTUtil::arrayGet($params, 'i18n_domain');
		if (empty($sDomain)) {
			$sDomain = $smarty->get_template_vars('i18n_domain');
		}
		if (empty($sDomain)) {
			$sDomain = 'knowledgeTree';
		}
		$oRegistry =& KTi18nRegistry::getSingleton();
		$oi18n =& $oRegistry->geti18n($sDomain);
		$content = trim($content);
		$replace = array(
		'@ *[\n\r]+@' => ' ',
		);
		$content = preg_replace(array_keys($replace), array_values($replace), $content);

		$content = $oi18n->gettext($content);
		return str_replace(array_keys($replacements), array_values($replacements), $content);
	}

	function addQueryStringBlock($params, $content, &$smarty, &$repeat) {
		if (empty($content)) {
			return;
		}
		return KTUtil::addQueryStringSelf($content);
	}

	function entity_select ($params, &$smarty) {
		require_once $smarty->_get_plugin_filepath('function', 'html_options');

		$entities = KTUtil::arrayGet($params, 'entities', array());
		/*
		if (is_null($entities)) {
		$smarty->trigger_error("assign: missing 'entities' parameter");
		return;
		}
		*/

		$method = KTUtil::arrayGet($params, 'method', 'getName');
		$none = KTUtil::arrayGet($params, 'none');

		$params['values'] = array();
		$params['output'] = array();
		if ($none) {
			$params['values'][] = '';
			$params['output'][] = 'None';
		}
		foreach ($entities as $oEntity) {
			$params['values'][] = $oEntity->getId();
			$params['output'][] = call_user_func(array(&$oEntity, $method));
		}
		unset($params['entities']);

		return smarty_function_html_options($params, $smarty);
	}

	function boolean_checkbox ($params, &$smarty) {
		$name = KTUtil::arrayGet($params, 'name');
		if (empty($name)) {
			$smarty->trigger_error("assign: missing 'name' parameter");
			return;
		}
		$bool = KTUtil::arrayGet($params, 'bool');
		if (is_null($bool)) {
			$smarty->trigger_error("assign: missing 'bool' parameter");
			return;
		}
		$value = KTUtil::arrayGet($params, 'value', 1);

		$label = KTUtil::arrayGet($params, 'label');

		if ($bool) {
			$checked = ' checked="checked"';
		} else {
			$checked = '';
		}

		$ret = sprintf('<input type="checkbox" name="%s" value="%s"%s />', $name, $value, $checked);
		if ($label) {
			$ret = sprintf('<label>%s%s</label>', $ret, $label);
		}
		return $ret;
	}

	function entity_checkboxes($params, &$smarty) {
		require_once $smarty->_get_plugin_filepath('function', 'html_checkboxes');

		$entities = KTUtil::arrayGet($params, 'entities');
		if (is_null($entities)) {
			$smarty->trigger_error("assign: missing 'entities' parameter");
			return;
		}

		$idmethod = KTUtil::arrayGet($params, 'idmethod', 'getId');
		$method = KTUtil::arrayGet($params, 'method', 'getName');

		$params['values'] = array();
		$params['output'] = array();
		foreach ($entities as $oEntity) {
			$params['values'][] = call_user_func(array(&$oEntity, $idmethod));
			if ($method != "none") {
				$params['output'][] = ' ' . call_user_func(array(&$oEntity, $method));
			} else {
				$params['output'][] = null;
			}
		}
		unset($params['entities']);
		unset($params['idmethod']);
		unset($params['method']);

		return smarty_function_html_checkboxes($params, $smarty);
	}

	function entity_radios($params, &$smarty) {
		require_once $smarty->_get_plugin_filepath('function', 'html_radios');

		$entities = KTUtil::arrayGet($params, 'entities');
		if (is_null($entities)) {
			$smarty->trigger_error("assign: missing 'entities' parameter");
			return;
		}

		$idmethod = KTUtil::arrayGet($params, 'idmethod', 'getId');
		$method = KTUtil::arrayGet($params, 'method', 'getName');

		$params['values'] = array();
		$params['output'] = array();
		foreach ($entities as $oEntity) {
			$params['values'][] = call_user_func(array(&$oEntity, $idmethod));
			$params['output'][] = call_user_func(array(&$oEntity, $method));
		}
		unset($params['entities']);
		unset($params['idmethod']);
		unset($params['method']);

		return smarty_function_html_radios($params, $smarty);
	}

	function addQueryString($url, $qs) {
		return KTUtil::addQueryString($url, $qs);
	}


	/*
	* ktLink generates a fully prepared link for KT.
	*
	* It takes into account kt_path_info requirements,
	* rootUrl requirements, etc.
	*
	* To do this effectively, you need three things:
	*
	*   @param base              e.g. /admin.php
	*   @param subpath           e.g. /security/permissions/
	*   @param query             e.g. fFolder=1&fMyVar=2
	*
	*/
	function ktLink($params, &$smarty) {
		return KTUtil::ktLink($params['base'], $params['subpath'], $params['query']);
	}


	function addQueryStringSelf($qs) {
		return KTUtil::addQueryStringSelf($qs);
	}

	function getUrlForFolder($params, &$smarty) {
		return KTBrowseUtil::getUrlForFolder($params['folder']);
	}

	function getCrumbStringForDocument($params, &$smarty) {
		$aBreadcrumbs = KTBrowseUtil::breadcrumbsForDocument($params['document'], array('final'=>true));
		if(PEAR::isError($aBreadcrumbs)) {
			return _kt('No breadcrumbs available');
		}

		$aCrumbs = array();
		foreach($aBreadcrumbs as $aBreadcrumb) {
			$aCrumbs[] = $aBreadcrumb['name'];
		}

		return implode('/', $aCrumbs);
	}


}

/*
NewsWire: определение новостного канала, построенного на основе
системы NewsBoy, разработанной Justin Grant
(Web: jusgrant.cjb.net or justin.host.za.net
Mail: justin@glendale.net)
*/
class LMXmlParser
{
	var $xml_parser;
	var $xml_file;
	var $data;
	var $_last_name;

	// Конструктор класса
	function LMXmlParser ()
	{
		$this->xml_parser = "";
		$this->xml_file = "";
		$this->html = "";
	}
	// Деструктор класса. В отличие от Си++ и Perl его
	// необходимо вызывать вручную
	function destroy()
	{
		xml_parser_free( $this->xml_parser);
	}

	// Вызывается при начале нового элемента в документе
	function startElement( $parser, $name, $attrs)
	{
	    // $this->data[$name] = '';
		$this->_last_name = $name;
	}

	// Вызывается при окончании обработки элемента
	function endElement ($parser , $name )
	{
		$this->_last_name = '';
	    // do nothing yet
	}
	// Обработчик символьных данных
	function characterData( $parser, $data )
	{
		$data = trim($data);
	    if (!empty($this->_last_name) && !empty($data)) {
		    $this->data[$this->_last_name][] = $data;			
		}
	}
	//
	// Разбор файла
	//
	function parse()
	{
		$this-> xml_parser = xml_parser_create() ;
		xml_set_object($this->xml_parser , $this) ;
		// обратите внимание на нивелирование регистров тегов
		// что позволяет при любом раскладе установить соответствие
		// их записям $map_array
		xml_parser_set_option($this->xml_parser,
		XML_OPTION_CASE_FOLDING, true );
		xml_set_element_handler ($this->xml_parser,
		"startElement" , "endElement" );
		xml_set_character_data_handler ($this->xml_parser,
		"characterData") ;
		if (!($fp = fopen($this->xml_file, "r"))) {
			return new PEAR_Error("He могу открыть входной файл XML ".$this->xml_file);
		}

		while ($data = fread( $fp, 4096))
		{
			if ( !xml_parse($this->xml_parser , $data, feof($fp)))
			{
				return new PEAR_Error(sprintf ("Ошибка в XML файле : %s на строке %d",
				xml_error_string( xml_get_error_code(
				$this->xml_parser->
				xml_get_current_line_number($this->xml_parser)) )));
			}
		}
	}

}

?>