<?php 
/**
 *
 * Represents a document link as per the database document_link table.
 * Links a child document to a parent document.  Direction of relationship
 * is defined through document collaboration.
 *
 * The contents of this file are subject to the KnowledgeTree Public
 * License Version 1.1 ("License"); You may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.ktdms.com/KPL
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * The Original Code is: KnowledgeTree Open Source
 * 
 * The Initial Developer of the Original Code is The Jam Warehouse Software
 * (Pty) Ltd, trading as KnowledgeTree.
 * Portions created by The Jam Warehouse Software (Pty) Ltd are Copyright
 * (C) 2006 The Jam Warehouse Software (Pty) Ltd;
 * All Rights Reserved.
 *
 *
 * @author Bryn Divey, Jam Warehouse (Pty) Ltd, South Africa
 * @package lib.documentmanagement 
 */
require_once(KT_LIB_DIR . '/documentmanagement/DocumentTransaction.inc');

class LMDocumentTransaction extends DocumentTransaction
{
    var $iCount;

    /**
     * Static function.  Given a document transaction primary key will create
     * a document transaction object and populate it with the corresponding
     * database values
     *
     * @return DocumentTransaction populated DocumentTransaction object on success, false otherwise and set $_SESSION["errorMessage"]
     */
    function & get($iDocumentTransactionID) {
        global $default, $lang_err_doc_not_exist;
        if (strlen($iDocumentTransactionID) > 0) {
            $sql = $default->db;
            //$sql->query(array("SELECT * FROM $default->document_transactions_table WHERE id = ?", $iDocumentTransactionID));/*ok*/
            $sql->query(array("SELECT dt.*, count(*) AS count FROM $default->document_transactions_table dt GROUP BY document_id, transaction_namespace HAVING id = ?", $iDocumentTransactionID));/*ok*/
            if ($sql->next_record()) {
                $oDocumentTransaction = & new LMDocumentTransaction(Document::get($sql->f("document_id")), $sql->f("comment"), $sql->f("transaction_namespace"));
                $oDocumentTransaction->iId = $sql->f("id");
                $oDocumentTransaction->sVersion = $sql->f("version");
                $oDocumentTransaction->iUserID = $sql->f("user_id");
                $oDocumentTransaction->dDateTime = $sql->f("datetime");
                $oDocumentTransaction->sIP = $sql->f("ip");
                $oDocumentTransaction->sFileName = $sql->f("filename");
                $oDocumentTransaction->bAdminMode = $sql->f("admin_mode");
                $oDocumentTransaction->iCount = $sql->f("count");
                return $oDocumentTransaction;
            }
            $_SESSION["errorMessage"] = $lang_err_object_not_exist . "id = " . $iDocumentTransactionID . " table = document_transactions";
            return false;
        } else {
            $_SESSION["errorMessage"] = "document transaction ID not set.  Cannot retrieve document with no id";
            return false;
        }
    }

    function getCount() { return $this->iCount; }

    function getList($sWhereClause = null) {
        return KTEntityUtil::getList2('LMDocumentTransaction', $sWhereClause);
    }


    function getStatByDocumentOld($oDocument) {
        $iDocumentId = KTUtil::getId($oDocument);
        return LMDocumentTransaction::getList(array('document_id = ?', array($iDocumentId)));
    }

    function getStatByDocument($oDocument) {
        global $default, $lang_err_folder_not_exist;

        $iDocumentId = KTUtil::getId($oDocument);

        /* @var $sql DBcompat */

        $db = $default->db;


        $query = array(array("SELECT dt.*, count(*) AS count
                        FROM $default->document_transactions_table dt 
                        WHERE dt.document_id = ? 
                        GROUP BY document_id, transaction_namespace ", array(intval($iDocumentId))));

        $res = DBUtil::getResultArray($query, $db);

        // return '<pre>'.print_r($res, true).'</pre>';
        return $res;
    }

    function getStatByFolder($oFolder, $bRecurse = true) {
        global $default, $lang_err_folder_not_exist;

        $iFolderId = KTUtil::getId($oFolder);

        /* @var $sql DBcompat */

        $db = $default->db;

        $params = array();
        $where = 'true';

        if ($iFolderId == 1) {
            if ($bRecurse) {
                $where = "parent_folder_ids LIKE '1,%'
                      OR
                      parent_folder_ids LIKE '1'";        
            } else {
                $where = "parent_folder_ids LIKE '1'";
            }
        } else {
            if ($bRecurse) {
                $where = "parent_folder_ids LIKE '%,?,%'
                      OR
                      parent_folder_ids LIKE '%,?'
                      OR
                      folder_id = ?";        
                $params = array(intval($iFolderId),intval($iFolderId),intval($iFolderId));

            } else {
                $where = "folder_id = ?";
                $params = array(intval($iFolderId));
            }
        }

        $query = array(array("SELECT dt.*, count(*) AS count
                            FROM document_transactions dt
                            WHERE dt.document_id IN
                            (
                            SELECT id FROM `documents`
                            WHERE ".
        $where
        ." )
                            GROUP BY transaction_namespace", $params));

        $res = DBUtil::getResultArray($query, $db);

        // return '<pre>'.print_r($res, true).'</pre>';
        return $res;

        //return LMDocumentTransaction::getList(array('document_id = ?', array($iDocumentId)));
    }
    
    function getDocDetailsByFolder($oFolder, $bRecurse = true) {
        global $default, $lang_err_folder_not_exist;

        $iFolderId = KTUtil::getId($oFolder);

        /* @var $sql DBcompat */

        $db = $default->db;

        $params = array();
        $where = 'true';

        if ($iFolderId == 1) {
            if ($bRecurse) {
                $where = "parent_folder_ids LIKE '1,%'
                      OR
                      parent_folder_ids LIKE '1'";        
            } else {
                $where = "parent_folder_ids LIKE '1'";
            }
        } else {
            if ($bRecurse) {
                $where = "parent_folder_ids LIKE '%,?,%'
                      OR
                      parent_folder_ids LIKE '%,?'";        
                $params = array(intval($iFolderId),intval($iFolderId));

            } else {
                $where = "parent_folder_ids LIKE '%,?'";
                $params = array(intval($iFolderId));
            }
        }

        $query = array(array("SELECT dt.*, d.full_path, dmv.name, count(*) AS count
                            FROM document_transactions dt
                            INNER JOIN documents d ON dt.document_id = d.id
                            INNER JOIN document_metadata_version dmv ON d.metadata_version_id = dmv.id
                            WHERE dt.transaction_namespace = 'ktcore.transactions.download' AND dt.document_id IN
                            (
                            SELECT id FROM `documents`
                            WHERE ".
        $where
        ." )
                            GROUP BY dt.document_id, transaction_namespace 
                            ORDER BY count DESC", $params));
        
        $res = DBUtil::getResultArray($query, $db);

        // return '<pre>'.print_r($res, true).'</pre>';
        return $res;

        //return LMDocumentTransaction::getList(array('document_id = ?', array($iDocumentId)));
    }

}

?>
