<?php

/**
 * $Id$
 *
 * The contents of this file are subject to the KnowledgeTree Public
 * License Version 1.1 ("License"); You may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.ktdms.com/KPL
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * The Original Code is: KnowledgeTree Open Source
 * 
 * The Initial Developer of the Original Code is The Jam Warehouse Software
 * (Pty) Ltd, trading as KnowledgeTree.
 * Portions created by The Jam Warehouse Software (Pty) Ltd are Copyright
 * (C) 2006 The Jam Warehouse Software (Pty) Ltd;
 * All Rights Reserved.
 *
 */

require_once(KT_LIB_DIR . '/actions/documentaction.inc.php');
require_once(KT_LIB_DIR . '/subscriptions/Subscription.inc');
require_once(KT_LIB_DIR . '/widgets/fieldWidgets.php');
require_once(KT_LIB_DIR . '/browse/browseutil.inc.php');
require_once(KT_LIB_DIR . '/documentmanagement/documentutil.inc.php');

function getVocabluary ($id = null){
	$voc = array(
		1 => _kt('Получить консультацию'),
		2 => _kt('Распечатка документа'),
		3 => _kt('Сообщить о проблеме'),
		4 => _kt('Другое'),
	);
	
	if ($id === null) {
		return $voc;
	}elseif (intval($id) > 0 && array_key_exists($id, $voc)) {
		return $voc[$id];
	} 
	return '';
}

class LMRequestPlugin extends KTPlugin {
	var $sNamespace = "lm.request.plugin";
	var $iOrder = 50;
	var $rVersion = 0.2;

	function LMRequestPlugin($sFilename = null) {
		$res = parent::KTPlugin($sFilename);
		$this->sFriendlyName = _kt('LiberMedia Document Request Plugin v'.$this->rVersion);
		return $res;
	}

	function setup() {
		$this->registerAction('documentaction', 'LMDocumentRequestAction','lm.request.request');
		$this->registerNotificationHandler('LMRequestNotification', 'libermedia/request', 'LMRequestPlugin.php');

		$oTemplating =& KTTemplating::getSingleton();
		$oTemplating->addLocation('libermedia', 'plugins/libermedia/templates');
	}

}

$oRegistry =& KTPluginRegistry::getSingleton();
$oRegistry->registerPlugin('LMRequestPlugin', 'lm.request.plugin', __FILE__);

// {{{ LMRequestAction
class LMDocumentRequestAction extends KTDocumentAction {
	var $sName = 'lm.request.request';

	function getDisplayName() {
		return _kt('Попросить помощь');
	}

	function do_main() {
		$this->oPage->setBreadcrumbDetails(_kt("Запрос"));
		$oTemplate =& $this->oValidator->validateTemplate('request');
		
		$aOptions = array(
			'vocab' => getVocabluary(), 
			'multi' => false,
			'size' => 1
		);
		
		$aFields = array(
			new KTLookupWidget(_kt('Цель'), _kt('Цель обращения'), 
							   'subject', "1", $this->oPage, true, null, null, $aOptions),
		    new KTStringWidget(_kt('Email для связи'), _kt('Вы можете указать email в персональных настройках, чтобы не вводить его вручную.'),
							   'email', $this->oUser->getEmail(), $this->oPage, true),
			new KTStringWidget(_kt('Тел. номер для связи'), _kt('Вы можете указать номер в персональных настройках, чтобы не вводить его вручную каждый раз.'),
							   'mobile', $this->oUser->getMobile(), $this->oPage, false),
			new KTTextWidget(_kt('Подробности'), _kt('Дополнительная информация по запросу'), 
								'details', "", $this->oPage, true),
		);
		
		$oTemplate->setData(array(
			'context' => &$this,
			'fields' => $aFields,
		));
		return $oTemplate->render();
	}

	function do_request() {
		$data =array('subject', 'mobile', 'details', 'email');
		$aRequest = array();
		foreach ($data as $val) {
			$aRequest[$val] = htmlentities(($_REQUEST[$val]));
		}
		
		$aUsers = $aGroups = $aRoles = array();
	
		foreach (Group::getAdministratorGroups() as $oGroup) {
			$aGroups[$oGroup->getId()] =& $oGroup;
		}

		foreach (Unit::getUnitsForFolder($this->oDocument->getFolderId()) as $oUnit) {
			foreach (Group::getUnitAdministratorGroupsByUnit($oUnit) as $oGroup) {
				$aGroups[$oGroup->getId()] =& $oGroup;
			}
		}

		$aRoles[-2] = Role::get(-2);
		$oDocument =& $this->oDocument;

		foreach ($aRoles as $oRole) {
			// Ignore anonymous or Everyone roles
			$iRoleId = KTUtil::getId($oRole);
			if (($iRoleId == -3) || ($iRoleId == -4)) {
				continue;
			}
			// first try on the document, then the folder above it.
			$oRoleAllocation = DocumentRoleAllocation::getAllocationsForDocumentAndRole($oDocument->getId(), $iRoleId);
			if (is_null($oRoleAllocation)) {
				// if we don't get a document role, try folder role.
				$oRoleAllocation = RoleAllocation::getAllocationsForFolderAndRole($oDocument->getFolderID(), $oRole->getId());
			}
			if (is_null($oRoleAllocation) || PEAR::isError($oRoleAllocation)) {
				continue;
			}
			$aRoleUsers = $oRoleAllocation->getUsers();
			$aRoleGroups = $oRoleAllocation->getGroups();

			foreach ($aRoleUsers as $id => $oU) {
				$aUsers[$id] = $oU;
			}
			foreach ($aRoleGroups as $id => $oGroup) {
				$aGroups[$id] = $oGroup;
			}
		}


		$aGroupMembershipSet = GroupUtil::buildGroupArray();
		$aAllIds = array_keys($aGroups);
		foreach ($aGroups as $id => $oGroup) {
			$aAllIds = kt_array_merge($aGroupMembershipSet[$id], $aAllIds);
		}

		foreach ($aAllIds as $id) {
			if (!array_key_exists($id, $aGroups)) {
				$aGroups[$id] = Group::get($id);
			}
		}

		// now, merge this (again) into the user-set.
		foreach ($aGroups as $oGroup) {
			$aNewUsers = $oGroup->getMembers();
			foreach ($aNewUsers as $oU) {
				$id = $oU->getId();
				if (!array_key_exists($id, $aUsers)) {
					$aUsers[$id] = $oU;
				}
			}
		}

		foreach ($aUsers as $oU) {
			if (!PEAR::isError($oU)) {
				LMRequestNotification::newNotificationForDocument($this->oDocument, $oU, $this->oUser, $aRequest);
			}
		}


		$this->commitTransaction();
		$params = 'fDocumentId=' . $oDocument->getId();
		$url = generateControllerLink('viewDocument', $params);
		exit(redirect($url));
	}
}
// }}}

class LMRequestNotification extends KTNotificationHandler {
	function & clearNotificationsForDocument($oDocument) {
		$aNotifications = KTNotification::getList('data_int_1 = ' . $oDocument->getId());
		foreach ($aNotifications as $oNotification) {
			$oNotification->delete();
		}
	}

	// isorokin: $oActor - это пользователь, отправивший запрос
	function &newNotificationForDocument($oDocument, $oUser, $oActor, $aRequest) {
		$aInfo = array(
			'sData1' => serialize(array(
				'iSubject' => $aRequest['subject'], 
				'sMobile' => $aRequest['mobile'],
				'sEmail' => $aRequest['email'],
			)),
			'sText1' => $aRequest['details'],
			'iData1' => $oDocument->getId(),
			'iData2' => $oActor->getId(),
			'sType' => 'libermedia/request',
			'dCreationDate' => getCurrentDateTime(),
			'iUserId' => $oUser->getId(),
			'sLabel' => $oDocument->getName(),
		);

		$oNotification = KTNotification::createFromArray($aInfo);

		$handler = new LMRequestNotification();
		$email = ( trim($aRequest['email']) > 0 ) ? trim($aRequest['email']) : $oUser->getEmail();
 
		if ($oUser->getEmailNotification() && (strlen($email) > 0)) {
			$emailContent = $handler->handleNotification($oNotification);
			$emailSubject = sprintf(_kt('Assistance request: %s'), $oDocument->getName());
			$oEmail = new EmailAlert($email, $emailSubject, $emailContent);
			$oEmail->send();
		}

		return $oNotification;
	}

	function handleNotification($oKTNotification) {
		$oTemplating =& KTTemplating::getSingleton();
		$oTemplate =& $oTemplating->loadTemplate('request_notification');
		$oTemplate->setData($this->getNotificationData($oKTNotification));
		return $oTemplate->render();
	}
	
	function notify_main() {
		$oTemplating =& KTTemplating::getSingleton();
		$oTemplate =& $oTemplating->loadTemplate('request_notification_details');
		$oTemplate->setData($this->getNotificationData());
		return $oTemplate->render();
	}

	function getNotificationData($oKTNotification = null)
	{
		$oKTNotification = (is_a($oKTNotification,'KTNotification')) 
				? $oKTNotification : $this->oNotification;
		if(! is_a($oKTNotification,'KTNotification')) {
			return false;
		}
		
		$aData1 = unserialize($oKTNotification->getStrData1());
		$oDoc = Document::get($oKTNotification->getIntData1());
		return array(
			'context' => $this,
			'document_id' => $oKTNotification->getIntData1(),
			'subject' => getVocabluary($aData1['iSubject']),
			'mobile' => $aData1['sMobile'],
			'email' => $aData1['sEmail'],
			'actor' => User::get($oKTNotification->getIntData2()),
			'document_name' => $oKTNotification->getLabel(),
			'notify_id' => $oKTNotification->getId(),
			'details' => $oKTNotification->getTextData1(),
			'document' => $oDoc,
			'is_broken' => (PEAR::isError($oDoc) || ($oDoc->getStatusID() != LIVE)),
		);
	}

	function resolveNotification($oKTNotification) {
		$notify_action = KTUtil::arrayGet($_REQUEST, 'notify_action', null);
		$this->oNotification =& $oKTNotification;
		$this->redispatch('notify_action', 'notify');
		exit(0);
	}

	function notify_clear() {
		$_SESSION['KTInfoMessage'][] = _kt('Assistance Request cleared.');
		$this->oNotification->delete();
		exit(redirect(generateControllerLink('dashboard')));
	}

	function notify_view() {
		$params = 'fDocumentId=' . $this->oNotification->getIntData1();
		$url = generateControllerLink('viewDocument', $params);
		// $this->oNotification->delete(); // clear the alert.
		exit(redirect($url));
	}
}

?>
