<?php

/**
 * $Id$
 *
 * Copyright (c) 2006 Jam Warehouse http://www.jamwarehouse.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; using version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 *
 * You can contact the copyright owner regarding licensing via the contact
 * details that can be found on the KnowledgeTree web site:
 *
 *         http://www.ktdms.com/
 */

require_once(KT_LIB_DIR . '/plugins/plugin.inc.php');
require_once(KT_LIB_DIR . '/plugins/pluginregistry.inc.php');
require_once(KT_LIB_DIR . '/browse/advancedcolumns.inc.php');

class MoreViewColumnsPlugin extends KTPlugin {
    var $sNamespace = "more_view_columns.plugin";
    
    function MoreViewColumnsPlugin($sFilename = null) {
        $res = parent::KTPlugin($sFilename);
        $this->sFriendlyName = _kt('More View Columns Plugin');
        return $res;
    }            
    
    function setup() {
        $this->registerColumn(_kt("Автор"), 'ktcore.columns.document_autor', 'DocumentAutorColumn', '../ktcore/KTColumns.inc.php');                
        $this->registerColumn(_kt("Idnotice"), 'ktcore.columns.document_idnotice', 'DocumentIdnoticeColumn', '../ktcore/KTColumns.inc.php');                
        $this->registerColumn(_kt("Заглавие"), 'ktcore.columns.document_header', 'DocumentHeaderColumn', '../ktcore/KTColumns.inc.php');                
        $this->registerColumn(_kt("Безопасный просмотр"), 'ktcore.columns.document_secview', 'DocumentSecviewColumn', '../ktcore/KTColumns.inc.php');                
        $this->registerColumn(_kt("Год издания"), 'ktcore.columns.document_date', 'DocumentDateColumn', '../ktcore/KTColumns.inc.php');                
    }
}

/**
 * класс переписан и выполняет функции абстрактного класса.
 * Для создания новой колонки надо создать унаследованный класс от данного
 * объявить уинкальные $sColName, $namespace 
 * и переписать renderData($aDataRow) - возврат отображаемого значения в колонке.
 * Для отображения в списке возможных столбцов добавить строку в setup()
 * и в управлении плагинами нажать update.
 * ВАЖНО: недопустимо объявлять имена классов и namespace если таковые уже существуют
 *
 */
class DocumentFileNameColumn extends AdvancedColumn {

    var $namespace = 'ktcore.columns.document_file_name';
    var $sColName;
    var $sortable = true;
    var $sort_on = true;
    
    
    function DocumentFileNameColumn() {
        $this->label = _kt($this->sColName);
    }

    function renderData($aDataRow) { 
        // only _ever_ show this for documents.
        if ($aDataRow["type"] === "folder") { 
            return '&nbsp;';
        }
        $link = KTBrowseUtil::getUrlForFolder($aDataRow['folder']);
        return sprintf('%s', $aDataRow["document"]->getFilename());
    }
    
    /**
     * @param string $sField - назваине поля 
     * @param array $oDocument
     * @return string - значение поля
     */
    function _getValueByField($aDataRow, $sField){
    	 if ($aDataRow["type"] === "folder") { 
            return '&nbsp;';
        }
    	$oFieldSet = KTFieldset::getByNamespace('lm.fieldsets.absotheque');
        $oField = DocumentField::getByFieldsetAndName($oFieldSet, $sField);
        $oFieldsLink = DocumentFieldLink::getByDocumentAndField($aDataRow["document"], $oField);
        if (!is_a($oFieldsLink, 'ktentitynoobjects'))
        {
        	return $oFieldsLink -> getValue();
        } else {
        	return '&nbsp;';
        }
    }

    function getName() { return _kt($this->sColName); }
}

/**
 * пример класса с новой колонкой "авторы"
 *
 */
class DocumentAutorColumn extends  DocumentFileNameColumn {
	
	var $sColName = 'Авторы';
	var $namespace = 'ktcore.columns.document_autor';
	
     function renderData($aDataRow) { 
      	$aAutors = explode(";",$this->_getValueByField($aDataRow, $this->sColName));
      	return $aAutors[0];
     }

}

class DocumentIdnoticeColumn extends  DocumentFileNameColumn {
	
	var $sColName = 'Idnotice';
	var $namespace = 'ktcore.columns.document_idnotice';
	
     function renderData($aDataRow) {
       	return $this->_getValueByField($aDataRow, 'id');
     }

}

class DocumentHeaderColumn extends  DocumentFileNameColumn {
	
	var $sColName = 'Заглавие';
	var $namespace = 'ktcore.columns.document_header';
	
     function renderData($aDataRow) {
       	return $this->_getValueByField($aDataRow, $this->sColName);
     }

}

class DocumentDateColumn extends  DocumentFileNameColumn {
	
	var $sColName = 'Год издания';
	var $namespace = 'ktcore.columns.document_date';
	
     function renderData($aDataRow) {
       	return $this->_getValueByField($aDataRow, $this->sColName);
     }

}


class DocumentSecviewColumn extends  DocumentFileNameColumn {
	var $sColName = 'Безопасный просмотр';
	var $namespace = 'ktcore.columns.document_secview';
	
	// переопределим renderData 
	function renderData($aDataRow) { 
        if ($aDataRow["type"] === "folder") { 
            return '&nbsp;';
        }
        //FIXME это криво 
      	$link = 'action.php?kt_path_info=ktcore.SecViewPlugin.actions.document&fDocumentId='.$aDataRow["document"]->iId;
      	$image = '<IMG src="plugins/SecView/img/edit-find.png">';
      	return sprintf('%s', "<a href='$link'>".$image."</a>");
    }
	
}
//rre
$oPluginRegistry =& KTPluginRegistry::getSingleton();
$oPluginRegistry->registerPlugin('MoreViewColumnsPlugin', 'more_view_columns.plugin', __FILE__);
?>