<?php

/**
 * $Id$
 *
 * Copyright (c) 2006 Jam Warehouse http://www.jamwarehouse.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; using version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 *
 * You can contact the copyright owner regarding licensing via the contact
 * details that can be found on the KnowledgeTree web site:
 *
 *         http://www.ktdms.com/
 */

require_once(KT_LIB_DIR . "/ktentity.inc");
require_once('QuicklinkLocation.inc.php');

class Quicklink extends KTEntity {
	var $iTargetId;
	var $bIsFolder;
	var $iPosition;
    var $bActive = false;
    var $iLocation;
    var $iLocationIsFolder;
    var $iLocationId;

    // this _aFieldToSelect array maps the names of the variables to 
    // columns in the database.
    var $_aFieldToSelect = array(
        "iId" => "id",
        "iTargetId" => "target_id",
        "bIsFolder" => "is_folder",
	    "iPosition" => "position",
	    "bActive" => "active",
	    "iLocation" => "location",
	    "iLocationId" => "location_id",
	    "bLocationIsFolder" => "location_is_folder",
    );

    // this specifies that errors from ->update(), ->create(), ->delete() and friends 
    // should be PEAR::raiseErrors.  This is recommended for ALL new code. 
    var $_bUsePearError = true;

    function getTargetId() { return $this->iTargetId; }
    function getIsFolder() { return $this->bIsFolder; }
    function getPosition() { return $this->iPosition; }
	function getActive() { return $this->bActive; }
	function getLocation() { return $this->iLocation; }
    function getLocationIsFolder() { return $this->bLocationIsFolder; }
    function getLocationId() { return $this->iLocationId; }

    function setTargetId($iTargetId) { $this->iTargetId = $iTargetId; }
    function setIsFolder($bIsFolder) { $this->bIsFolder = $bIsFolder; }
    function setPosition($iPosition) { $this->iPosition = $iPosition; }
    function setActive($bActive) { $this->bActive = $bActive; }
    function setLocation($iLocation) { $this->iLocation = $iLocation; }
    function setLocationIsFolder($bLocationIsFolder) { $this->bLocationIsFolder = $bLocationIsFolder; }
    function setLocationId($iLocationId) { $this->iLocationId = $iLocationId; }

    function _table() { return KTUtil::getTableName('quicklinks'); }

    // FIXME: This function should disappear as soon as I find the way to
    // be notified of deleted folders.
	function isValid() { 
	    $target = $this->getTarget();
		if (PEAR::isError($target))
		    return false;
		
		if ($iLocationId == QUICKLINK_LOC_DOCUMENT ||
		    $iLocationId == QUICKLINK_LOC_FOLDER)
		{
		    $location = $this->getLinkLocation();
		    if (PEAR::isError($location))
		        return false; 
		}
		
		return true;
	}

	function getTarget() {
	    if($this->getIsFolder()) {
		    $oTarget = Folder::get($this->getTargetId());
		} else {
		    $oTarget = Document::get($this->getTargetId());
		}
	    return $oTarget;
	}
	
	function getLinkLocation() {
	    if($this->getLocationIsFolder()) {
		    $oTarget = Folder::get($this->getLocationId());
		} else {
		    $oTarget = Document::get($this->getLocationId());
		}
	    return $oTarget;
	}

	function getTargetName() {
	    $oTarget = $this->getTarget();
		if (PEAR::isError($oTarget)) { return _('Object no longer exists'); }
	    return $oTarget->getName();
	}
	
	function getLocationName() {
	    $oTarget = $this->getLinkLocation();
		if (PEAR::isError($oTarget)) { return _('Object no longer exists'); }
	    return $oTarget->getName();
	}
	
	function getContentType() {
	    if($this->getIsFolder()) return "folder";
	    $oTarget = $this->getTarget();
		return KTMime::getIconPath($oTarget->getMimeTypeId());
	}
	
	function getLocationContentType() {
	    if($this->getLocationIsFolder()) return "folder";
	    $oTarget = $this->getLinkLocation();
		return KTMime::getIconPath($oTarget->getMimeTypeId());
	}	
	
	function getTargetQueryString() {
	    if($this->getIsFolder()) { $sTargetType = "Folder"; }
		else { $sTargetType = "Document"; }
		return sprintf("f%sId=%s", $sTargetType, $this->getTargetId());
	}
	
	function getLocationQueryString() {
	    if($this->getLocationIsFolder()) { $sTargetType = "Folder"; }
		else { $sTargetType = "Document"; }
		return sprintf("f%sId=%s", $sTargetType, $this->getLocationId());
	}
	
	function getLocationString() {
	    return QuicklinkLocation::getStringByLocation($this->getLocation());
	}

    // STATIC
    function &get($iId) { 
    	return KTEntityUtil::get('Quicklink', $iId); 
    }
    
    function &createFromArray($aOptions) { 
    	return KTEntityUtil::createFromArray('Quicklink', $aOptions); 
//    	$oKTEntityUtil  = new KTEntityUtil();
//    	return $oKTEntityUtil->createFromArray('Quicklink', $aOptions); 
    }
    
    function &getList($sWhereClause = null, $aOptions = null) {
        if (is_null($aOptions)) { $aOptions = array(); }
        $aOptions['orderby'] = KTUtil::arrayGet($aOptions, 'orderby', 'position');
        return KTEntityUtil::getList2('Quicklink', $sWhereClause, $aOptions);
    }
	
	function &getAtPosition($iPosition) {
	    $aQuicklinks = Quicklink::getList(sprintf("position = %d", $iPosition));
		if(!count($aQuicklinks)) return PEAR::raiseError("No quicklink found at position", null, PEAR_ERROR_RETURN);
		return $aQuicklinks[0];
	}
	
	function getNextPosition() {
	    $aQuicklinks = Quicklink::getList();
        if (PEAR::isError($aQuicklinks)) return $aQuicklinks;
	    if (empty($aQuicklinks)) { return 0; }
		return $aQuicklinks[count($aQuicklinks) - 1]->getPosition() + 1;
	}
}

?>
