<?php
/**
 * $Id$
 *
 * The contents of this file are subject to the KnowledgeTree Public
 * License Version 1.1.2 ("License"); You may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.knowledgetree.com/KPL
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific language governing rights and
 * limitations under the License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by KnowledgeTree" logo and
 *    (ii) the KnowledgeTree copyright notice
 * in the same form as they appear in the distribution.  See the License for
 * requirements.
 * 
 * The Original Code is: KnowledgeTree Open Source
 * 
 * The Initial Developer of the Original Code is The Jam Warehouse Software
 * (Pty) Ltd, trading as KnowledgeTree.
 * Portions created by The Jam Warehouse Software (Pty) Ltd are Copyright
 * (C) 2007 The Jam Warehouse Software (Pty) Ltd;
 * All Rights Reserved.
 * Contributor( s): David Alfonso
 */

// Use this constant to enable/disable permission checking
define('QUICKLINK_OPT_CHECK_PERMISSION', 1);

require_once(KT_LIB_DIR . '/plugins/plugin.inc.php');
require_once(KT_LIB_DIR . '/plugins/pluginregistry.inc.php');
require_once(KT_LIB_DIR . '/database/sqlfile.inc.php');
require_once(KT_LIB_DIR . '/database/dbutil.inc');
require_once('adminManageQuicklinks.php');


class QuicklinksPlugin extends KTPlugin {
    var $sNamespace = "bd.Quicklinks.plugin";

    
    function QuicklinksPlugin($sFilename = null) {
        $res = parent::KTPlugin($sFilename);
        $this->sFriendlyName = _kt('Quicklinks Plugin');
        return $res;
    }

    function setup() {
        global $default;
        
        // Dashlets
        $this->registerDashlet('QuicklinksDashlet', 
        					'bd.Quicklinks.dashlet', 
        					'QuicklinksDashlet.php');
        
        // Pages
        $this->registerAdminPage("quicklinksmanagement", 
			'ManageQuicklinksDispatcher', 
			'misc',
			_kt('Manage Quicklinks'), 
			_kt('Allows you to modify the quicklinks shown in the dashboard, in folders, in documents or in search results.'),
			'adminManageQuicklinks.php', 
			null);
					
		// Triggers
		$this->registerTrigger('delete', 'postValidate', 
							'QuicklinksDeleteDocumentTrigger', 
				        	'bd.Quicklinks.triggers.delete');

        // Templates
        require_once(KT_LIB_DIR . "/templating/templating.inc.php");
        $oTemplating =& KTTemplating::getSingleton();
        $oTemplating->addLocation('QuicklinksDashlet', '/plugins/quicklinks/templates');

        // Collection Columns
        $this->registerColumn('QuicklinkSingleSelectionColumn', 
                              'bd.Quicklinks.columns.selection', 
                              'QuicklinkSingleSelectionColumn', 
                              'adminManageQuicklinks.php');
        
        // Portlet
        $this->registerPortlet(array('browse', 'view'), 'QuicklinksPortlet',
                               'bd.Quicklinks.portlets.quicklinks',
                               'QuicklinksPortlet.php');
        
        $default->quicklinks_table = 'quicklinks';
    }
	
	function register() {
	    $oEnt = parent::register();
		// can be: PEAR::Error [failed to reg. ], oEntity (already reg.), true (first reg)
		if (PEAR::isError($oEnt)) { 
		    return $oEnt; 
		}
		
		if (!$this->_detectTables()) {
		    $res = $this->_createTables();
			if (PEAR::isError($res)) { 
			    return $res;
			}
		}
		
		return $oEnt;
	}
	
	// detect the tables 
	function _detectTables() {
	    $sTable = KTUtil::getTableName('quicklinks');
	    $sQuery = 'SELECT * FROM ' . $sTable;
		$aQuery = array($sQuery, array());
		
		$res = DBUtil::runQuery($aQuery);
		
		//var_dump($res);
		return (!PEAR::isError($res)); // error = false, success = true
	}
	
	function _createTables() {
	    global $default;
		
	    $sLocalDir = dirname(__FILE__);
	    $aQueries = SQLFile::sqlFromFile($sLocalDir . '/sql/install.sql');
		
		// create the admin connection
		DBUtil::setupAdminDatabase();
		$db = $default->_admindb;
		
		
		// use a transaction - all or nothing (warning:  may not work for CREATE () ENGINE MyISAM
		DBUtil::startTransaction();
		$res = DBUtil::runQueries($aQueries, $db);
		if (PEAR::isError($res)) {
		    DBUtil::rollback();
			return $res;
		}
		DBUtil::commit();
		return true;
	}
	
}

class QuicklinksDeleteDocumentTrigger {
    var $aInfo = null;
    
    function setInfo(&$aInfo) {
        $this->aInfo =& $aInfo;
    }

    function postValidate() {
        $oDocument =& $this->aInfo["document"];
        
        // Delete the document from quicklinks table (if it exists).
        $sTable = KTUtil::getTableName('quicklinks');
	    $sQuery = 'DELETE FROM ' . $sTable . " WHERE target_id = ? AND is_folder = ?";
		$aQuery = array($sQuery, array($oDocument->getId(), 0));
		
		$res = DBUtil::runQuery($aQuery);
		
        // Check for errors
        if (PEAR::isError($res)) {
			return $res;
		}
		
		return true;
    }
}

$oPluginRegistry =& KTPluginRegistry::getSingleton();
$oPluginRegistry->registerPlugin('QuicklinksPlugin', 'bd.Quicklinks.plugin', __FILE__);
?>
