<?php

/**
 *
 * helpers.php,   The BitRock Update Server PHP API
 *
 * Methods to represent the notifications information
 *
 * @author BitRock www.BitRock.com
 *
 */

require_once 'message_store.php';

/**
 * Formats a date or shows an alternative message
 */
function last_checked_to_s($checked){
    return ($checked?date("Y-m-d H:i:s", $checked):"never");
}

/**
 * Shows the list of update server messages
 */
function showList(){
    $ud = MessageStore::singleton();
    $result = '';
    $result.= '<div class="last_checked">Last Checked: '.last_checked_to_s($ud->lastChecked()).'</div>';
    $result.= "<form target=\"".$_SERVER['PHP_SELF']."\" method=\"post\" name=\"UpdateClientForm\" id=\"UpdateClientForm\">";
    $result.= '<table width="95%" cellspacing="0">';            

    $messages = $ud->getNotifications();
    if (!empty($messages)) {
        foreach ( $messages as $m ){
            if ( !$m->isArchived() || (array_key_exists('update_client_do', $_GET) && $_GET['update_client_do']=='showall')) {
                $result .= showMessageEntry($m);
            }
        }
    }else{
        $result .= "No messages";
    }

    $result .= '</table><br/>';
    $result .= '<input type="hidden" id="update_client_do_selection" name="update_client_do_selection" value=""/>';
    $result .= '<div class="update_client_controls">';
//  $result .= '<a href="?update_client_do=refresh" onClick="UAM_show_list(\'refresh\');return false;" >refresh</a> | ';
    $result .= '<a href="?update_client_do=showall" onClick="UAM_show_list(\'showall\');return false;" >show all</a> | ';
    $result .= '<a href="" onclick="UAM_process_selection(\'keepnew\'); return false;">keep new</a> | ';    
    $result .= '<a href="" onclick="UAM_process_selection(\'markasread\'); return false;">mark as read</a> | ';
    $result .= '<a href="" onclick="UAM_process_selection(\'archive\');return false;">archive</a>';
//  $result .= ' | <a href="?update_client_do=delete_all" onClick="UAM_show_list(\'delete_all\');return false;" >delete all</a>';
    $result .= '</div></form>';
    return $result;
}


/**
 * Shows a message element in the list
 */ 
function showMessageEntry( $message ) {  
    $type = $message->getType();
    $sel = (array_key_exists('update_client_selection', $_POST) ? $_POST['update_client_selection'] : array());
    $result = '<tr><td><input name="update_client_selection[]" value="'.$message->getId().'" type="checkbox" ';
    if ($sel && is_array($sel) && in_array($message->getId(),$sel))
        $result.= ' checked ';
    $result.= '/></td><td ';
    if ($message->isRead())
        $result.= 'class="read">';
    else
        $result.= 'id="'.$type.($message->isCritical() ? "-critical":"" ).'" '.($message->isRead()?"class=read":"").'>';
    $result.= $type;
    if ( $message->isCritical())   $result .= '<b style="color:red;"> !</b>'; 

    $result .= '</td><td><a class="message_short" href="'.$_SERVER['PHP_SELF'].'?update_client_id='.$message->getId().'" ';
    $result .= ' onClick="UAM_show_message('.$message->getId().'); return false;">';
    $title = $message->getTitle();
    if ( $message -> isRead() ) {
        $result .= '<span class="read">'.substr($title,0,50).(strlen($title)>50?'...':'').'</span>';
    }else{
        $result .= '<b>'.substr($title,0,50).(strlen($title)>50?'...':'').'</b>';
    }
    $desc = $message ->getDescription();
    if (strlen($title) < 50 && $desc)
        $result .= '<span style="color:gray;">. '.substr($desc,0,50-strlen($title)).((strlen($desc)>50-strlen($title))?'...':'').'</span></a></td>';

    $result .= '<td '.($message->isRead()?"class=read":"").'>'.date("d/m/y",$message->getCreatedAt()).'</td></tr>';
    return $result;
}


/**
 * Shows the content of a specific message
 */
function showContent($id){
    $result = '';
    $ud = MessageStore::singleton();
    if ( $m = $ud->getNotification($id) ) {
        $result.= '<div class="details"><div id="'.$m->getType().'-details" width="95%">';                   
        $result.= showMessage($m).'</div></div>';
        $m -> markAsRead();
        $ud -> saveNotification($m);
        $result .= '<div class="update_client_controls">';
        $result.= '<a href="'.$_SERVER['PHP_SELF'].'?" onClick="UAM_show_list(); return false;" >back to list</a> | ';
        if ($m->isArchived()){
         $result .= '<a href="?update_client_do=not_archive&update_client_id='.$m->getId().
                '" onClick="UAM_not_archive_message('.$id.');return false;">mark as not archived</a> ';
        }else{
            $result .= '<a href="?update_client_do=archive&update_client_id='.$m->getId().
                '" onClick="UAM_archive_message('.$id.');return false;">archive</a> ';
        }
        $result .= '</div>';
    }          
    return $result;
}


/**
 * Shows the details of a message
 */
function showMessage( $message ) {
    $result = '<p><b>';
    if ( $message->isCritical() ) 
        $result.= '<img src="/plugins/updatesandmessages/images/critical.png" height="16px" alt="critical!" /> ';    
    $result .= $message->getTitle().'</b>&nbsp;&nbsp;<small>'.date("Y-m-d H:i:s",$message->getCreatedAt()).'</small></p>';
    $result .= '<p>'.html_entity_decode($message->getDescription()).'</p>';

    if ( $web = $message->getWebpageUrl() ){        
        if ( $desc = $message->getWebpageUrlDescription() )
            $result .= '<p><a href="'.$web.'">'.$desc.'</a><p>';
        else
            $result .= '<p><a href="'.$web.'">webpage</a></p>';
    }

    if ( $message->isUpdate() ) {
        if ($message->getUpdateInstruction())
            $result .= '<p><small>INSTRUCTIONS</small><br/> '.$message->getUpdateInstruction().'</p>';
        if ($message->getCmdSwitch())                        
            $result .= '<small>COMMAND LINE SWITCHES:</small> '.$message->getCmdSwitch().'<br/>';
        if ($message->getUpdateType())
            $result .= '<small>TYPE:</small> '.$message->getUpdateType().'<br/>';  
        if ($message->getSize())
            $result .= '<small>SIZE:</small> '.$message->getSize().'<br/>';
        if ($message->getOs())
            $result .= '<small>OS:</small> '.$message->getOs().'<br/>';          
        if ($message->getMd5())
            $result .= '<small>MD5:</small> '.$message->getMd5().'<br/>';  
        if ($message->getUpdateUrl())
            $result .= '<a href="'.$message->getUpdateUrl().'">Get Update</a>';
    }

    return $result;
}


/**
 * Shows the content of a specific message
 */
function showNews( $message ) {      
    $result = '';      
    if ($message->getTitle())
        $result .= "<p><b>".$message->getTitle()."</b>&nbsp;&nbsp;<small>".$message->getCreatedAt()."</small></p>";
    if ($message->getDescription())
        $result .= "<p>".$message->getDescription()."</p>";

    if ( $web = $message->getUrl() ){        
        if ( $desc = $message->getUrlDescription() )
            $result .= '<p><a href="'.$web.'">'.$desc.'</a><p>';
        else
            $result .= '<p><a href="'.$web.'">webpage</a></p>';
    }

    $result .= '<div class="update_client_controls">';
    $result .= '<a href="?archive_news">mark as read</a></div>';
    return $result;
}
?>
