<?php

/**
 * Messages, The BitRock Update Server PHP API
 *
 * Implementation of ....
 *
 * @author BitRock www.BitRock.com
 * @version 0.1
 * @package MessageStore
 *
 */

require_once 'um_database.php';

/**
 * Class that describes a notification
 */
class Notification {
    protected $properties = array();
    protected $valid_properties = array('guid', 'type', 'title','description', 'critical',
      'webpage_url', 'webpage_url_description', 'update_md5', 'update_url', 'update_cmd_switch', 
      'update_instruction', 'update_size', 'update_type', 'update_os','is_read', 'is_archived');
    public $datetime_properties = array('created_at');
    public $boolean_properties  = array('critical');

    function __construct($notification) {      
        foreach ($notification as $p_key => $p_value) {           
            $this -> properties[$p_key] = $p_value;        
        }
    }

    /**
     * Functions to modify or get a property of this notification
     */
    function setProperty($name, $value) { $this -> properties[$name] = $value;            }
    function getProperty($name)         { return $this -> properties[$name];              }    
    function setProperties($properties) {             
        foreach ($properties as $p_key => $p_value) $this -> properties[$p_key] = $p_value;                       
    }

    /**
     * Helpers to obtain some of the properties
     */
    function getId()                    { return $this -> getProperty('id');              }    
    function setId($value)              { return $this -> setProperty('id', $value);      }
    function getGuid()                  { return $this -> getProperty('guid');            }
    function getTitle()                 { return $this -> getProperty('title');           }
    function getDescription()           { return $this -> getProperty('description');     }
    function getLanguage()              { return $this -> getProperty('language');        }
    function getWebpageUrl()            { return $this -> getProperty('webpage_url');     }
    function getWebpageUrlDescription() { return $this-> getProperty('webpage_url_description');}
    function isCritical()               { return ($this -> getProperty('critical') == 1); }
    function getType()                  { return $this -> getProperty('type');            }
    function isMessage()                { return ($this -> getType() == 'message');       }
    function isUpdate()                 { return ($this -> getType() == 'update');        }
    function isSystemEvent()            { return ($this -> getType() == 'system_event');  }
    function getCreatedAt()             {
        $created_at =  $this->getProperty('created_at');   
        return (is_string($created_at) ? strtotime($created_at) : $created_at); 
    }

    /**
     * Functions to manage the read or unread state of the notifications
     */
    function isRead()                   { return ($this -> getProperty('is_read') == 1);  }
    function markAsRead()               { $this -> setProperty('is_read', 1);             }
    function markAsNotRead()            { $this -> setProperty('is_read', 0);             }

    /**
     * Functions manage the archived property
     */
    function isArchived()               { return ($this -> getProperty('is_archived')==1);}
    function markAsArchived()           { $this -> setProperty('is_archived', 1);         }
    function markAsNotArchived()        { $this -> setProperty('is_archived', 0);         } 

    /**
     * Saves the current notification to the local database
     *    Receives a SQL database handler and the table name
     */
    function save($table){     
        $result = false;
        $database = UM_Database::singleton();
        if ($table){
            $keys   = array();
            $values = array();            
            foreach($this->properties as $key=>$value){
                if (is_string($key) && in_array($key, $this->valid_properties)){
                    $keys[] = $key;
                    if (in_array($key,$this->datetime_properties))
                        $values[] = date("'Y-m-d H:i:s'",strtotime($value));
                    else if (in_array($key, $this->boolean_properties))
                        in_array($value,array("true","false","1","0",1,0)) ? $values[] = $value : array_pop($keys);
                    else                  
                        $values[] = (is_numeric($value)?$value:"\"$value\"");
                }
            }            
            if(count($keys)!=1){
                $result = $database->query("INSERT INTO $table (".
                    implode(',', $keys).') VALUES('.
                    implode(',', $values).')');
                if ($result)
                     $this->setProperty('id',$database->last_inserted_id());
            }
        }
        return $result;
    }

    /**
     * Updates all the parameters or only the keys indicated of the current element in the database
     */
    function update($table, $keys=false){               
        $result = false;
        if ($this->getId() && $table){
            $database = UM_Database::singleton();
            $property_keys = ($keys?$keys:array_keys($this->properties));
            $new_values = array();
            foreach($property_keys as $k){
                if (is_string($k) && in_array($k, $this->valid_properties)){
                    $val = $this->properties[$k];
                    $new_values[] = ($val==null? "$k = NULL" : (is_numeric($val) ?"$k = $val":"$k = \"$val\""));
                }
            }               
            if (count($new_values)!=0)
                $result = $database->query(
                    "UPDATE ".$table." SET ".implode(',',$new_values)." WHERE id = ".$this->getID());     
        }
        return $result;
    }
}



/**
 * Class that describes an Update
 */
class Update extends Notification {
    /**
     * Helpers to obtain some of the properties
     */  
    function getUpdateUrl()           {  return $this -> getProperty('update_url');        }        
    function getSize()                {  return $this -> getProperty('update_size');       }
    function getMd5()                 {  return $this -> getProperty('update_md5');        }
    function getCmdSwitch()           {  return $this -> getProperty('update_cmd_switch');  }
    function getUpdateInstruction()   {  return $this -> getProperty('update_instruction');}
    function getUpdateType()          {  return $this -> getProperty('update_type');       }
    function getOs()                  {  return $this -> getProperty('update_os');         }

    /**
     * Functions to manage the download state of the update
     */  
    function isDownloaded()           {  return ($this -> getProperty('is_downloaded')==1);}
    function markAsDownloaded()       {  return $this -> setProperty('is_downloaded',1);   }
    function markAsNotDownloaded()    {  return $this -> setProperty('is_downloaded',0);   }

    /**
     * Functions to manage the application of the update
     */
    function isApplied()              {  return ($this -> getProperty('is_applied')==1);   }
    function markAsAplied()           {  return $this -> getProperty('is_applied',1);      }
    function markAsNotAplied()        {  return $this -> getProperty('is_applied',0);      }
}


/**
 * Class that describes a Message
 */ 
class Message extends Notification {
}

/**
 * Class that describes a SystemEvent
 */
class SystemEvent extends Notification {
}


/**
 * Class that describes a short message
 **/
class ShortMessage extends Notification {
    protected $valid_properties = array('title','message', 'url', 'url_description', 'status', 'is_archived' );    
    
    function getStatus()               {  return $this -> getProperty('status');                 }
    function setStatus($value)         {  return $this -> setProperty('status', $value);         }
    function getDescription()          {  return $this -> getProperty('message');                }
    function setDescription($value)    {  return $this -> getProperty('message',$value);         }
    function getURL()                  {  return $this -> getProperty('url');                    }
    function setURL($value)            {  return $this -> setProperty('url', $value);            }
    function getURLDescription()       {  return $this -> getProperty('url_description');        }
    function setURLDescription($value) {  return $this -> setProperty('url_description', $value);}    
    function getType()                 {  return 'short_message';                                }

    function isActive(){ return UM_Config::$take_last_notification_as_news || $this->getStatus() == UM_Config::$visible_news_status; }
    function markAsRead(){}
    function markAsNotRead(){}  
    
    function setDefaultProperties(){
        $id = $this->getId();
        foreach(array_keys($this->properties) as $key)            
            $this->properties[$key] = null;
        $this->setStatus(0);
        $this->markAsNotArchived();        
        $this->setId($id);
    }
    
    /**
    * Modifies the stored short message with the last news received
    */
    function updateNews($news){     
        $result = false;        
        if (!$this->sameNews($news)){
            $this->setDefaultProperties();
            $this->setProperties($news);
            $result = $this->update(UM_Config::$news_table);
        }
        return $result;
    }
    
    /**
    * Detects if the received news are the same that are stored
    */
    private function sameNews($news){
        $same = true;       
        
        if (array_key_exists('title', $news)  ? ($news['title'] != $this->getTitle())     : !empty($this->properties['title']))
           $same = false;                
        if (array_key_exists('message', $news)? ($news['message'] != $this->getDescription())       : !empty($this->properties['message']))
           $same = false;
        if (array_key_exists('status', $news) ? (intval($news['status']) != $this->getStatus()) : !empty($this->properties['status']))
           $same = false;
        if (array_key_exists('url', $news)    ? ($news['url'] != $this->getURL())               : !empty($this->properties['url']))
           $same = false;
        if (array_key_exists('url_description', $news) ? ($news['url_description'] != $this->getURLDescription()) : !empty($this->properties['url_description']))
           $same = false;
        
        return $same; 
    }    
}

?>
