<?php
  /**
   * MessageStore, The BitRock Update Server PHP API
   *
   * Implementation of ....
   *
   * @author BitRock www.BitRock.com
   * @version 0.1
   * @pacage MessageStore
   *
   * @todo
   */

require_once 'um_config.php';
require_once 'um_database.php';
require_once 'message_xml.php';   // Provides ReadXMLMessages
require_once 'message.php';       // Provides Notification, Message, Update, SystemEvent

class MessageStore {
    private static  $instance;  
    private         $notice = ''; // Contains a simple text message that will be showed to the user
    private         $debug  = ''; // Contains debug messages that will be stored inside a hidden div

    private function __construct() {
    //   $this -> configuration = new Configuration();         
    }

    public function __clone() {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    public static function singleton() {
        if (!isset(self::$instance)) {
            $c = __CLASS__;
            self::$instance = new $c;
        }
        return self::$instance;
    }

    /**
     * Get all Notifications (any kind)     
     */
    function getNotifications($conditions = '') {
        $database = UM_Database::singleton();
        $res = $database->array_query('SELECT * FROM '.UM_Config::$notifications_table.
                ($conditions == ''?'':" WHERE $conditions").
                ' ORDER BY created_at DESC');
        $result = array();
        if ($res!=false){
            foreach ($res as &$m) {
                switch ($m['type']) {
                case 'update':
                    $result[] = new Update($m);
                    break;
                case 'message':
                    $result[] = new Message($m);
                    break;
                case 'system_event':
                    $result[] = new SystemEvent($m);
                    break;
                default:
                    $result[] = new Notification($m);
                }
            }        
        }
        return $result;
    }

    /**
     * Get Messages, Updates, SystemEvents ...
     *
     * Return only those notifications with Message, Update or SystemEvent type ...
     */
    function getMessages()     { return $this -> getNotifications('type = "message"');     }
    function getUpdates()      { return $this -> getNotifications('type = "update"');      }
    function getSystemEvents() { return $this -> getNotifications('type = "system_event"');}

    /**
     * Get Notification by id     
     */
    function getNotification($notification_id) {
        $result = $this -> getNotifications("id = ".$notification_id);
        return $result[0];
    }

    /**
     * Saves a notification in the database
     */
    function saveNotification($notification) {
        $notification->update(UM_Config::$notifications_table,array('is_read','is_archived'));
    }

    /**
    * Returns the last short message stored in the database
    */
    function getNews(){
        $database = UM_Database::singleton();
        $res = $database->array_query('SELECT * FROM '.(UM_Config::$take_last_notification_as_news ?
               UM_Config::$notifications_table.' SORT BY created_at DESC' : 
               UM_Config::$news_table).' LIMIT 1');                
        if ($res && count($res)!=0)
            $result = new ShortMessage($res[0]);
        else
            $result = false;        
        return $result;        
    }

     /**
     * Obtains a list of guid from the messages stored in the database
     */
    function getGuids($restriction=''){
        $guids = array();
        $database = UM_Database::singleton();
        $res = $database->array_query('SELECT guid FROM '.UM_Config::$notifications_table.
            ($restriction==''?'':" where $restriction"));
        if ($res)
            foreach( $res as $q) 
                if (array_key_exists('guid',$q))
                    $guids[] = $q['guid'];                            
        return $guids;
    }
    function getReadGuids()     { return $this->getGuids('is_read = TRUE'); }
    function getArchivedGuids() { return $this->getGuids('is_archived = TRUE'); }
    
    function getMessageGuids()  { return $this->getGuids('type = "message"'); }      
    function getMessageReadGuids()  { return $this->getGuids('type = "message" AND is_read = TRUE'); }
    function getMessageArchivedGuids()  { return $this->getGuids('type = "message" AND is_archived = TRUE'); }
    function getUpdateGuids()   { return $this->getGuids('type = "update"'); }
    function getUpdateArchivedGuids()   { return $this->getGuids('type = "update"  AND is_archived = TRUE'); }    
    function getUpdateReadGuids()   { return $this->getGuids('type = "update"  AND is_read = TRUE'); }
    
    /**
     * Stores new messages using their 'guid' to determine if a message is new or not
     */
    private function update_stored_notifications($info, $previous_guids, $remove_not_received=false){
        $received_guids = array();
        $guids_to_be_removed = array();
      
        if ($info && $info->info){
            $messages = $info->getMessages();
            foreach($messages as $message){
                if (isset($message['guid'])){
                    $guid = intval($message['guid']);                    
                    if (!in_array($guid, $previous_guids) && (!array_key_exists('status', $message) || intval($message['status']) != UM_Config::$removed_status)){
                        $received_guids[] = $guid;
                        $n = new Notification($message);
                        $n->save(UM_Config::$notifications_table);
                    } else if (array_key_exists('status', $message) && intval($message['status']) == UM_Config::$removed_status){
                        $guids_to_be_removed[] = $guid;
                    } else {
                        $received_guids[] = $guid;      
                    }
                }
            }         
            $this->update_lastchecked();        
            if ($remove_not_received && is_string($info->info->tag)){
                $type_of_xml = trim(strtolower($info->info->tag));                
                $conditions  = '';
                switch ($type_of_xml){
                    case "notifications":
                        $conditions = "type IN ('update','message')";
                        break; 
                    case "updates":
                        $conditions = "type = 'update'";
                        break;
                    case "messages":
                        $conditions = "type = 'message'";
                        break;
                    default:
                        $conditions = "type = '".substr($type_of_xml,0,-1)."'";
                        break;
                }
                $this->remove_notifications($received_guids, false, $conditions);         
            }
            if (!empty($guids_to_be_removed)){
                $this->remove_notifications($guids_to_be_removed);
            }
        }
        return $received_guids;
    }   

    /**
     * Removes the notifications that aren't in the list
     */ 
    private function remove_notifications($list_of_guids, $in_list=true, $conditions=''){
        $cond = array();
        $database = UM_Database::singleton();
        $q = 'DELETE FROM '.UM_Config::$notifications_table;    
        if (!empty($list_of_guids))
            $cond[] = 'guid '.($in_list ? '' : 'NOT ').'IN('.join(",", $list_of_guids).')';
        if (!empty($conditions))
            $cond[] = $conditions;
        if (!empty($cond))
            $q .= ' WHERE '.join(' AND ', $cond);        
        $database->query($q);
    }    
    
    /**
    * Replaces current stored short message with the one received replacing or removing the previous one
    */
    private function update_stored_news($info){
        $result = '';      
        if ($info){
            $news = $info->getOneLevelArray();
            $previous = $this->getNews();
          
            if (!$previous){        
                $news = new ShortMessage($news);
                $news->save(UM_Config::$news_table);
            }else{
                $previous->updateNews($news);
            }
        }
    }
    
    /**
    * Hides the last option received in simple dashlet    
    */
    public function archiveNews(){
        $n = $this->getNews();
        if ($n){
            $n->markAsArchived();
            $n->update(UM_Config::$news_table, array('is_archived'));
        }
    }
    
    
    /**
    * Uses the xml information received to modify notifications or short messages
    */
    public function update_stored_info($xml, $remove_not_received){
        $info = new ReadXMLMessages($xml);
        $main_tag = trim(strtolower($info->getMainTag()));
        switch($main_tag){
          case "messages":
          case "updates":
          case "notifications":
              $previous_guids = $this->getGuids();
              $this->update_stored_notifications($info, $previous_guids, $remove_not_received);
          break;          
          case "answer":              
              $this->update_stored_news($info);              
        } 
    }
    
    /**
     * Sets last checked to current time
     */
    function update_lastchecked(){    
        $database = UM_Database::singleton();
        $database->query('UPDATE '.UM_Config::$config_table.' SET last_checked = "'.date('Y-m-d H:i:s',Time()).'"');
    }

    /**
     * Obtains the date of the last server check for messages
     */
    function lastChecked(){
        $lc = '';
        $database = UM_Database::singleton();
        $query    = $database->array_query('SELECT last_checked FROM '.UM_Config::$config_table.' LIMIT 1');
        if ($query && $query[0])               
            $lc = strtotime($query[0]['last_checked']);
        return $lc;
    }

    /**
     * Try to refresh notifications and get new ones from the Update Agent/Server
     *
     * if there is a problem with the Update Server then proper values will be set in the hash returned by getInfo() function.
     *
     * @return integer $status 0 - OK, 1 - error
     */
    function refresh($delete_not_received=false) {   
        return 0;
    }          

    /**
     * Gets a notice message
     */
    function getNotice(){
        $not = $this->notice;
        $this->notice = '';
        return $not;
    }
}

?>
